@@ -19,11 +19,21 @@ rstan_load_time <- as.POSIXct("1970-01-01 00:00.00 UTC")
 RNG <- 0
 OUT <- 0
 
+tbbmalloc_proxyDllInfo <- NULL
+
 .onLoad <- function(libname, pkgname) {
-  assignInMyNamespace("rstan_load_time", value = Sys.time())  
+  assignInMyNamespace("rstan_load_time", value = Sys.time())
   set_rstan_ggplot_defaults()
   assignInMyNamespace("RNG", value = get_rng(0))
   assignInMyNamespace("OUT", value = get_stream())
+  Rcpp::loadModule("class_model_base", what = TRUE)
+  Rcpp::loadModule("class_stan_fit", what = TRUE)
+  ## the tbbmalloc_proxy is not loaded by RcppParallel which is linked
+  ## in by default on macOS
+  if(Sys.info()["sysname"] == "Darwin") {
+      tbbmalloc_proxy  <- system.file("lib/libtbbmalloc_proxy.dylib", package="RcppParallel", mustWork=FALSE)
+      tbbmalloc_proxyDllInfo <<- dyn.load(tbbmalloc_proxy, local = FALSE, now = TRUE)
+  }
 }
 
 .onAttach <- function(...) {
@@ -35,8 +45,22 @@ OUT <- 0
                         "options(mc.cores = parallel::detectCores()).\n",
                         "To avoid recompilation of unchanged Stan programs, we recommend calling\n",
                         "rstan_options(auto_write = TRUE)")
-  if (.Platform$OS.type == "windows")
-    packageStartupMessage("For improved execution time, we recommend calling\n",
-                          "Sys.setenv(LOCAL_CPPFLAGS = '-march=corei7 -mtune=corei7')\n",
-                          "although this causes Stan to throw an error on a few processors.")
+  if (.Platform$OS.type == "windows") {
+    R_version <- as.integer(R.version$major)
+    processor_msg <- ifelse(R_version >= 4, 
+                            "Sys.setenv(LOCAL_CPPFLAGS = '-march=native -mtune=native')", 
+                            "Sys.setenv(LOCAL_CPPFLAGS = '-march=corei7 -mtune=corei7')")
+    packageStartupMessage("For improved execution time, we recommend calling\n", processor_msg)
+  }
+}
+
+.onUnload <- function(libpath) {
+   # unload the package library
+   library.dynam.unload("rstan", libpath)
+
+   # unload tbbmalloc_proxy if we loaded it
+   if (!is.null(tbbmalloc_proxyDllInfo)) {
+       dyn.unload(tbbmalloc_proxyDllInfo[["path"]])
+       tbbmalloc_proxyDllInfo <<- NULL
+   }
 }
